from collections.abc import Callable, Mapping
from dataclasses import Field
from typing import (
    TYPE_CHECKING,
    Any,
    ClassVar,
    NamedTuple,
    Protocol,
    TypeVar,
    runtime_checkable,
)

if TYPE_CHECKING:
    from .parser import ConfigParser


type FieldDeserializer = Deser1 | Deser2
type Deser1 = Callable[[Any], Any]
type Deser2 = Callable[[Any, ConfigParser], Any]

type FieldSerializer = Callable[[Any], Any]


ConfigSpec = NamedTuple(
    "ConfigSpec",
    [
        ("top_level", bool),
        ("name", str),
        ("field_mappings", Mapping[str, str]),
        ("field_deserialzers", Mapping[str, FieldDeserializer]),
        ("field_serializers", Mapping[str, FieldSerializer]),
    ],
)

_T_co = TypeVar("_T_co", covariant=True)


@runtime_checkable
class ConfigClass(Protocol[_T_co]):
    __dataclass_fields__: ClassVar[dict[str, Field[Any]]]
    __config__: ClassVar[ConfigSpec]



from pathlib import Path
from typing import Any

import rtoml

from .config_format import ConfigFormat


class TOMLFormat(ConfigFormat):
    """Read and write TOML configuration files."""

    def __init__(self, none_value: str | None = "null") -> None:
        """
        Args:
            none_value: controls how `None` values are serialized.
                `none_value=None` means `None` values are ignored.
        """
        self.none_value = none_value

    def read(self, path: Path) -> dict[str, dict[str, Any]] | None:
        if not path.exists():
            return None

        return rtoml.load(path, none_value=self.none_value)

    def write(self, path: Path, data: dict[str, dict[str, Any]]) -> None:
        rtoml.dump(data, path, none_value=self.none_value)



from __future__ import annotations

import json
from pathlib import Path
from typing import Any

from .config_format import ConfigFormat


class JSONFormat(ConfigFormat):
    """Read and write JSON configuration files."""

    def read(self, path: Path) -> dict[str, dict[str, Any]] | None:
        if not path.exists():
            return None

        with path.open("r", encoding="utf-8") as file:
            return json.load(file)

    def write(self, path: Path, data: dict[str, dict[str, Any]]) -> None:
        with path.open("w", encoding="utf-8") as file:
            json.dump(data, file, indent=2)



from dataclasses import is_dataclass


def is_configclass_type(tp: type) -> bool:
    if not is_dataclass(tp):
        return False
    return hasattr(tp, "__config__") and hasattr(tp, "__dataclass_fields__")



from .decorator import configclass
from .exceptions import InvalidConfigClassError, MultipleTopLevelConfigError
from .parser import ConfigParser
from .types import FieldDeserializer, FieldSerializer

__all__ = [
    configclass,
    ConfigParser,
    FieldDeserializer,
    FieldSerializer,
    InvalidConfigClassError,
    MultipleTopLevelConfigError,
]



from collections.abc import Mapping
from dataclasses import MISSING, fields, is_dataclass
import inspect
from pathlib import Path
import types
from typing import (
    Any,
    TypeGuard,
    TypeVar,
    Union,
    cast,
    get_args,
    get_origin,
    get_type_hints,
)

from .format.config_format import ConfigFormat
from .format.toml_format import TOMLFormat
from .types import ConfigClass, Deser1, Deser2, FieldDeserializer
from .utils import is_configclass_type
from .exceptions import MultipleTopLevelConfigError, InvalidConfigClassError

T = TypeVar("T")


def wants_parser(d: FieldDeserializer) -> TypeGuard[Deser2]:
    try:
        sig = inspect.signature(d)
    except (TypeError, ValueError):
        return True

    params = list(sig.parameters.values())

    positional = [
        p
        for p in params
        if p.kind
        in (inspect.Parameter.POSITIONAL_ONLY, inspect.Parameter.POSITIONAL_OR_KEYWORD)
    ]
    has_varargs = any(p.kind == inspect.Parameter.VAR_POSITIONAL for p in params)
    return has_varargs or len(positional) >= 2


def apply_deserializer(d: FieldDeserializer, value: Any, parser: "ConfigParser") -> Any:
    if wants_parser(d):
        return d(value, parser)
    return cast(Deser1, d)(value)


class ConfigParser:
    """Parse config files into registered dataclasses."""

    def __init__(
        self,
        config_path: Path | str,
        *configs: type[Any],
        create_noexist: bool = False,
        format: ConfigFormat | None = None,
    ) -> None:
        """
        Args:
            config_path: path to the config file.
            *configs: config classes to include in this parser.
            create_noexist: if True, write defaults when missing.
            format: file format handler (defaults to TOML).
        """

        if format is None:
            format = TOMLFormat()
        config_path = Path(config_path)

        if not create_noexist and not config_path.exists():
            raise FileNotFoundError("Provided config file does not exist")

        self._config_path = config_path
        self._format = format

        self._add_configs(*configs)
        self._config = self._read_config()

    def _add_configs(self, *configs: type[ConfigClass]) -> None:
        invalid_configs: list[type] = []
        toplevel_configs: list[type] = []

        for cls in configs:
            if not is_configclass_type(cls):
                invalid_configs.append(cls)
            elif cls.__config__.top_level:
                toplevel_configs.append(cls)

        if invalid_configs:
            invalid = ", ".join(cls.__name__ for cls in invalid_configs)
            raise InvalidConfigClassError(
                f"Config classes must use @configclass: {invalid}"
            )

        if len(toplevel_configs) > 1:
            toplevel = ", ".join(cls.__name__ for cls in toplevel_configs)
            raise MultipleTopLevelConfigError(
                f"Only one top-level config class is allowed; found: {toplevel}"
            )

        self._configs = sorted(
            configs,
            key=lambda cls: (
                0 if cls.__config__.top_level else 1,
                cls.__config__.name.casefold(),
            ),
        )

    def _read_config(self) -> dict[str, Any]:
        config_data = self._format.read(self._config_path)
        if config_data is None:
            return self._create_default_config()

        return config_data

    def _create_default_config(self) -> dict[str, Any]:
        config_data: dict[str, Any] = {}

        for config_class in self._get_root_configs():
            config_class_data = ConfigParser._get_class_fields(config_class)

            if config_class.__config__.top_level:
                config_data = config_data | config_class_data
            else:
                section_name = config_class.__config__.name
                config_data[section_name] = config_class_data

        self._format.write(self._config_path, config_data)

        return config_data

    @staticmethod
    def _get_class_fields(
        config_class: type[ConfigClass[Any]],
    ) -> dict[str, Any]:
        class_data: dict[str, Any] = {}

        field_mappings = config_class.__config__.field_mappings
        field_serializers = config_class.__config__.field_serializers
        resolved_types = get_type_hints(config_class, include_extras=True)

        for field in fields(config_class):
            key = field_mappings.get(field.name, field.name)
            field_type = ConfigParser._unwrap_optional(resolved_types[field.name])
            value = ConfigParser._get_field_default_value(field, field_type)

            value = ConfigParser._serialize_field_value(
                value, field.name, field_serializers
            )
            class_data[key] = value

        return class_data

    @staticmethod
    def _get_field_default_value(field: Any, field_type: Any) -> Any:
        if field.default is not MISSING:
            return field.default
        elif field.default_factory is not MISSING:
            return field.default_factory()
        elif is_dataclass(field_type):
            return ConfigParser._get_class_fields(cast(type[ConfigClass], field_type))
        return None

    @staticmethod
    def _serialize_field_value(
        value: Any, field_name: str, field_serializers: Mapping[str, Any]
    ) -> Any:
        if field_name in field_serializers:
            return field_serializers[field_name](value)
        elif isinstance(value, Path):
            return str(value)
        elif isinstance(value, ConfigClass):
            return ConfigParser._get_class_fields(type(value))
        return value

    @staticmethod
    def _unwrap_optional(field_type: Any) -> Any:
        origin = get_origin(field_type)
        if origin is Union or origin is types.UnionType:
            type_args = [t for t in get_args(field_type) if t is not type(None)]
            if type_args:
                return type_args[0]
        return field_type

    def _get_root_configs(self) -> list[type[ConfigClass]]:
        nested_configs = set[type[ConfigClass]]()
        for config_class in self._configs:
            resolved_types = get_type_hints(config_class, include_extras=True)
            for field in fields(config_class):
                field_type = ConfigParser._unwrap_optional(resolved_types[field.name])
                if is_configclass_type(field_type):
                    nested_configs.add(field_type)

        return [c for c in self._configs if c not in nested_configs]

    def _convert_field_value(self, value: Any, field_type: type) -> Any:
        if value is None:
            return None
        if is_dataclass(field_type):
            return self._parse_config(cast(type[ConfigClass], field_type), value)
        elif isinstance(field_type, type) and not isinstance(value, field_type):
            return field_type(value)
        return value

    def _parse_config(
        self, config_class: type[ConfigClass[T]], section_data: dict[str, Any]
    ) -> T:
        field_mappings = config_class.__config__.field_mappings
        field_deserializers = config_class.__config__.field_deserialzers
        kwargs = {}

        resolved_types = get_type_hints(config_class, include_extras=True)
        for field in fields(config_class):
            key = field_mappings.get(field.name, field.name)

            if key not in section_data:
                raise KeyError(
                    f"Missing config key '{key}' for '{config_class.__name__}'"
                )

            value = section_data[key]

            if field.name in field_deserializers:
                value = apply_deserializer(field_deserializers[field.name], value, self)
            else:
                field_type = ConfigParser._unwrap_optional(resolved_types[field.name])
                value = self._convert_field_value(value, field_type)

            kwargs[field.name] = value

        return cast(T, config_class(**kwargs))

    def add(self, *configs: type[Any]) -> None:
        self._add_configs(*configs, *self._configs)

    def get(self, config_class: type[T]) -> T:
        """
        Generic method to get any registered config.
        """

        if config_class not in self._configs:
            raise ValueError(
                f"Config class {config_class.__name__} was not provided to this parser"
            )

        cast_class = cast(type[ConfigClass[T]], config_class)

        if cast_class.__config__.top_level:
            config_data = self._config
            return self._parse_config(cast_class, config_data)

        section_name = cast_class.__config__.name
        if section_name not in self._config:
            raise KeyError(f"Missing {section_name} configuration from config file")

        config_data = self._config[section_name]
        return self._parse_config(cast_class, config_data)



from collections.abc import Callable, Mapping
from typing import (
    TypeVar,
    cast,
    overload,
)

from .types import ConfigClass, ConfigSpec, FieldDeserializer, FieldSerializer

_T = TypeVar("_T")


def _apply_config[_T](  # noqa: UP049
    cls: type[_T],
    *,
    top_level: bool,
    name: str,
    field_name_mappings: Mapping[str, str] | None,
    field_deserialzers: Mapping[str, FieldDeserializer] | None,
    field_serializers: Mapping[str, FieldSerializer] | None,
) -> type[_T]:
    resolved_name = name or cls.__name__
    if not top_level and resolved_name == "":
        raise ValueError("name can't be empty in a non-top-level package")

    cast(type[ConfigClass[_T]], cls).__config__ = ConfigSpec(
        top_level=top_level,
        name=resolved_name,
        field_mappings=field_name_mappings or {},
        field_deserialzers=field_deserialzers or {},
        field_serializers=field_serializers or {},
    )

    return cls


@overload
def configclass[_T](cls: type[_T]) -> type[_T]: ...  # noqa: UP049


@overload
def configclass(
    *,
    top_level: bool = False,
    name: str = "",
    field_name_mappings: Mapping[str, str] | None = None,
    field_deserialzers: Mapping[str, FieldDeserializer] | None = None,
    field_serializers: Mapping[str, FieldSerializer] | None = None,
) -> Callable[[type[_T]], type[_T]]: ...


def configclass[_T](  # noqa: UP049
    cls: type[_T] | None = None,
    *,
    top_level: bool = False,
    name: str = "",
    field_name_mappings: Mapping[str, str] | None = None,
    field_deserialzers: Mapping[str, FieldDeserializer] | None = None,
    field_serializers: Mapping[str, FieldSerializer] | None = None,
) -> type[_T] | Callable[[type[_T]], type[_T]]:
    if cls is not None:
        return _apply_config(
            cls,
            top_level=top_level,
            name=name,
            field_name_mappings=field_name_mappings,
            field_deserialzers=field_deserialzers,
            field_serializers=field_serializers,
        )

    def decorator(
        inner_cls: type[_T],
    ) -> type[_T]:
        return _apply_config(
            inner_cls,
            top_level=top_level,
            name=name,
            field_name_mappings=field_name_mappings,
            field_deserialzers=field_deserialzers,
            field_serializers=field_serializers,
        )

    return decorator



from dataclasses import dataclass, field
from pathlib import Path
from tempfile import TemporaryDirectory
from classconf import ConfigParser, configclass
from classconf.format import JSONFormat


@configclass
@dataclass
class Nested:
    value: int = 1


@configclass(top_level=True)
@dataclass
class AppConfig:
    name: str = "demo"
    nested: Nested = field(default_factory=Nested)


def main() -> None:
    with TemporaryDirectory() as tmp:
        path = Path(tmp) / "config.json"
        parser = ConfigParser(
            path,
            AppConfig,
            Nested,
            format=JSONFormat(),
            create_noexist=True,
        )
        config = parser.get(AppConfig)
        assert config.name == "demo"
        assert config.nested.value == 1


if __name__ == "__main__":
    main()



import json
import unittest
from abc import ABC
from dataclasses import dataclass, field
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any, Protocol, runtime_checkable

from classconf import (
    ConfigParser,
    configclass,
    InvalidConfigClassError,
    MultipleTopLevelConfigError,
)
from classconf.format import JSONFormat, TOMLFormat


class TestConfigParser(unittest.TestCase):
    def test_separate_classes_json_format(self) -> None:
        @configclass(name="alpha")
        @dataclass
        class AlphaConfig:
            label: str = "alpha"

        @configclass(name="beta")
        @dataclass
        class BetaConfig:
            count: int = 5

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                AlphaConfig,
                BetaConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            expected = json.dumps(
                {"alpha": {"label": "alpha"}, "beta": {"count": 5}}, indent=2
            )
            self.assertEqual(config_path.read_text(), expected)

            alpha = parser.get(AlphaConfig)
            beta = parser.get(BetaConfig)
            self.assertEqual(alpha.label, "alpha")
            self.assertEqual(beta.count, 5)

    def test_top_level_with_nested_types(self) -> None:
        @configclass
        @dataclass
        class PathsConfig:
            output_dir: Path = Path("./out")

        @configclass
        @dataclass
        class FlagsConfig:
            retries: int = 3
            verbose: bool = False

        @configclass(top_level=True)
        @dataclass
        class AppConfig:
            name: str = "demo"
            paths: PathsConfig = field(default_factory=PathsConfig)
            flags: FlagsConfig = field(default_factory=FlagsConfig)

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                AppConfig,
                PathsConfig,
                FlagsConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            config = parser.get(AppConfig)
            self.assertEqual(config.name, "demo")
            self.assertEqual(config.flags.retries, 3)
            self.assertFalse(config.flags.verbose)
            self.assertEqual(config.paths.output_dir, Path("./out"))

    def test_top_level_with_extra_section_toml(self) -> None:
        @configclass(top_level=True)
        @dataclass
        class MainConfig:
            title: str = "main"
            enabled: bool = True

        @configclass
        @dataclass
        class ExtraConfig:
            level: int = 2

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.toml"
            parser = ConfigParser(
                config_path,
                MainConfig,
                ExtraConfig,
                format=TOMLFormat(),
                create_noexist=True,
            )

            main = parser.get(MainConfig)
            extra = parser.get(ExtraConfig)
            self.assertEqual(main.title, "main")
            self.assertTrue(main.enabled)
            self.assertEqual(extra.level, 2)

    def test_nested_name_collision_allows_lookup(self) -> None:
        @configclass(name="nested")
        @dataclass
        class NamedNested:
            value: str = "value"

        @configclass(top_level=True)
        @dataclass
        class ParentConfig:
            nested: NamedNested = field(default_factory=NamedNested)

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                ParentConfig,
                NamedNested,
                format=JSONFormat(),
                create_noexist=True,
            )

            nested = parser.get(NamedNested)
            self.assertEqual(nested.value, "value")

        @configclass
        @dataclass
        class DefaultNested:
            value: str = "value"

        @configclass(top_level=True)
        @dataclass
        class ParentDefaultConfig:
            nested: DefaultNested = field(default_factory=DefaultNested)

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                ParentDefaultConfig,
                DefaultNested,
                format=JSONFormat(),
                create_noexist=True,
            )

            with self.assertRaises(KeyError):
                parser.get(DefaultNested)

    def test_missing_defaults_written_as_null(self) -> None:
        @configclass(top_level=True)
        @dataclass
        class RequiredConfig:
            count: int
            label: str

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                RequiredConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            self.assertEqual(
                json.loads(config_path.read_text()),
                {"count": None, "label": None},
            )
            config = parser.get(RequiredConfig)
            self.assertIsNone(config.count)
            self.assertIsNone(config.label)

    def test_serializers_apply_to_default_config(self) -> None:
        def serialize_count(value: int) -> str:
            return f"{value}x"

        @configclass(
            name="metrics",
            field_serializers={"count": serialize_count},
        )
        @dataclass
        class MetricsConfig:
            count: int = 3

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            ConfigParser(
                config_path,
                MetricsConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            self.assertEqual(
                json.loads(config_path.read_text()),
                {"metrics": {"count": "3x"}},
            )

    def test_deserializers_without_parser(self) -> None:
        def deserialize_num(value: str, **_: Any) -> int:
            return int(value.rstrip("x"))

        @configclass(
            name="metrics",
            field_deserialzers={"count": deserialize_num},
        )
        @dataclass
        class MetricsConfig:
            count: int

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            config_path.write_text(json.dumps({"metrics": {"count": "8x"}}))
            parser = ConfigParser(
                config_path,
                MetricsConfig,
                format=JSONFormat(),
                create_noexist=False,
            )

            config = parser.get(MetricsConfig)
            self.assertEqual(config.count, 8)

    def test_deserializers_using_parser_with_serialized_field(self) -> None:
        @runtime_checkable
        class Child(Protocol):
            name: str
            size: int

        @configclass(name="child")
        @dataclass
        class ChildConfig:
            name: str = "alpha"
            size: int = 2

        def resolve_child(_: Any, parser: ConfigParser) -> ChildConfig:
            return parser.get(ChildConfig)

        def serialize_child(child: ChildConfig) -> str:
            return child.name

        @configclass(
            top_level=True,
            field_deserialzers={"child": resolve_child},
            field_serializers={"child": serialize_child},
        )
        @dataclass
        class ParentConfig:
            child: Child = field(default_factory=ChildConfig)

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                ParentConfig,
                ChildConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            parent = parser.get(ParentConfig)
            self.assertEqual(parent.child.name, "alpha")
            self.assertEqual(parent.child.size, 2)

    def test_deserializers_using_parser_with_abc_child(self) -> None:
        class Child(ABC):
            name: str
            size: int

        @configclass(name="child")
        @dataclass
        class ChildConfig(Child):
            name: str = "alpha"
            size: int = 2

        def resolve_child(_: Any, parser: ConfigParser) -> ChildConfig:
            return parser.get(ChildConfig)

        def serialize_child(child: ChildConfig) -> str:
            return child.name

        @configclass(
            top_level=True,
            field_name_mappings={"child": "child_ref"},
            field_deserialzers={"child": resolve_child},
            field_serializers={"child": serialize_child},
        )
        @dataclass
        class ParentConfig:
            child: Child = field(default_factory=ChildConfig)

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                ParentConfig,
                ChildConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            parent = parser.get(ParentConfig)
            self.assertEqual(parent.child.name, "alpha")
            self.assertEqual(parent.child.size, 2)

    def test_default_format_is_toml(self) -> None:
        @configclass(top_level=True)
        @dataclass
        class DefaultFormatConfig:
            value: int = 1

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.toml"
            parser = ConfigParser(config_path, DefaultFormatConfig, create_noexist=True)

            self.assertIsInstance(parser._format, TOMLFormat)
            self.assertTrue(config_path.exists())

    def test_missing_file_raises(self) -> None:
        @configclass(top_level=True)
        @dataclass
        class MissingFileConfig:
            value: int = 1

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "missing.json"
            with self.assertRaises(FileNotFoundError):
                ConfigParser(config_path, MissingFileConfig, format=JSONFormat())

    def test_invalid_config_class_rejected(self) -> None:
        @dataclass
        class InvalidConfig:
            value: int = 1

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            with self.assertRaises(InvalidConfigClassError):
                ConfigParser(
                    config_path, InvalidConfig, format=JSONFormat(), create_noexist=True
                )

    def test_multiple_top_level_rejected(self) -> None:
        @configclass(top_level=True)
        @dataclass
        class TopOne:
            value: int = 1

        @configclass(top_level=True)
        @dataclass
        class TopTwo:
            value: int = 2

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            with self.assertRaises(MultipleTopLevelConfigError):
                ConfigParser(
                    config_path,
                    TopOne,
                    TopTwo,
                    format=JSONFormat(),
                    create_noexist=True,
                )

    def test_missing_key_raises(self) -> None:
        @configclass(name="app")
        @dataclass
        class RequiredConfig:
            count: int
            label: str

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            config_path.write_text(json.dumps({"app": {"count": 1}}))
            parser = ConfigParser(
                config_path,
                RequiredConfig,
                format=JSONFormat(),
                create_noexist=False,
            )

            with self.assertRaises(KeyError):
                parser.get(RequiredConfig)

    def test_add_includes_new_configs(self) -> None:
        @configclass(name="alpha")
        @dataclass
        class AlphaConfig:
            value: int

        @configclass(name="beta")
        @dataclass
        class BetaConfig:
            label: str

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            config_path.write_text(
                json.dumps({"alpha": {"value": 1}, "beta": {"label": "ok"}})
            )
            parser = ConfigParser(
                config_path,
                AlphaConfig,
                format=JSONFormat(),
                create_noexist=False,
            )

            parser.add(BetaConfig)
            beta = parser.get(BetaConfig)
            self.assertEqual(beta.label, "ok")

    def test_get_missing_config_class_raises(self) -> None:
        @configclass(name="alpha")
        @dataclass
        class AlphaConfig:
            value: int = 1

        @configclass(name="beta")
        @dataclass
        class BetaConfig:
            value: int = 2

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            parser = ConfigParser(
                config_path,
                AlphaConfig,
                format=JSONFormat(),
                create_noexist=True,
            )

            with self.assertRaises(ValueError):
                parser.get(BetaConfig)

    def test_missing_section_raises(self) -> None:
        @configclass(name="alpha")
        @dataclass
        class AlphaConfig:
            value: int = 1

        with TemporaryDirectory() as temp_dir:
            config_path = Path(temp_dir) / "config.json"
            config_path.write_text(json.dumps({"other": {"value": 1}}))
            parser = ConfigParser(
                config_path,
                AlphaConfig,
                format=JSONFormat(),
                create_noexist=False,
            )

            with self.assertRaises(KeyError):
                parser.get(AlphaConfig)


if __name__ == "__main__":
    unittest.main()



import unittest
from dataclasses import dataclass

from classconf import configclass

from .helpers import as_config_class


def deserialize_count(value: str) -> int:
    return int(value)


def serialize_count(value: int) -> str:
    return str(value)


class TestConfigClassDecorator(unittest.TestCase):
    def test_metadata_is_attached(self) -> None:
        @configclass(
            top_level=True,
            name="app",
            field_name_mappings={"count": "count_value"},
            field_deserialzers={"count": deserialize_count},
            field_serializers={"count": serialize_count},
        )
        @dataclass
        class ExampleConfig:
            count: int = 1

        config_class = as_config_class(ExampleConfig)
        spec = config_class.__config__

        self.assertTrue(spec.top_level)
        self.assertEqual(spec.name, "app")
        self.assertEqual(spec.field_mappings, {"count": "count_value"})
        self.assertEqual(spec.field_deserialzers, {"count": deserialize_count})
        self.assertEqual(spec.field_serializers, {"count": serialize_count})


if __name__ == "__main__":
    unittest.main()



from abc import ABC
from pathlib import Path
from typing import Any


from abc import abstractmethod


class ConfigFormat(ABC):
    @abstractmethod
    def read(self, path: Path) -> dict[str, Any] | None: ...

    @abstractmethod
    def write(self, path: Path, data: dict[str, Any]) -> None: ...
